@@ -491,7 +491,7 @@ extern void *valloc (size_t __size) __THROW __attribute_malloc__;
 #ifdef __USE_XOPEN2K
 /* Allocate memory of SIZE bytes with an alignment of ALIGNMENT.  */
 extern int posix_memalign (void **__memptr, size_t __alignment, size_t __size)
-     __THROW __attribute_malloc__;
+     __THROW;
 #endif
 
 __BEGIN_NAMESPACE_STD
@@ -768,13 +768,6 @@ extern int ptsname_r (int __fd, char *__buf, size_t __buflen) __THROW;
 extern int getpt (void) __THROW;
 #endif
 
-#ifdef __USE_BSD
-/* Put the 1 minute, 5 minute and 15 minute load averages into the first
-   NELEM elements of LOADAVG.  Return the number written (never more than
-   three, but may be less than NELEM), or -1 if an error occurred.  */
-extern int getloadavg (double __loadavg[], int __nelem) __THROW;
-#endif
-
 #endif /* don't just need malloc and calloc */
 #undef __need_malloc_and_calloc
 
