@@ -32,17 +32,15 @@ extern void __guard_setup(void);
 /*
  * Prototypes.
  */
-extern void *__libc_stack_end;
+void *__libc_stack_end;
 extern void weak_function _stdio_init(void);
-extern int * __errno_location(void);
-extern int * __h_errno_location(void);
+extern int *weak_const_function __errno_location(void);
+extern int *weak_const_function __h_errno_location(void);
 #ifdef __UCLIBC_HAS_LOCALE__
 extern void weak_function _locale_init(void);
 #endif
 #ifdef __UCLIBC_HAS_THREADS__
 extern void weak_function __pthread_initialize_minimal(void);
-extern int __pthread_linkin;
-
 #endif
 
 
@@ -133,9 +131,6 @@ void __uClibc_init(void)
     __pagesize = PAGE_SIZE;
 
 #ifdef __UCLIBC_HAS_THREADS__
-    
-    __pthread_linkin = 0;
-
     /* Before we start initialzing uClibc we have to call
      * __pthread_initialize_minimal so we can use pthread_locks
      * whenever they are needed.
