@@ -1,31 +1,10 @@
-/*  Copyright (C) 2002     Manuel Novoa III
+/* Copyright (C) 2002-2004   Manuel Novoa III    <mjn3@codepoet.org>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License as published by the Free Software Foundation; either
- *  version 2 of the License, or (at your option) any later version.
+ * GNU Library General Public License (LGPL) version 2 or later.
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public
- *  License along with this library; if not, write to the Free
- *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Dedicated to Toni.  See uClibc/DEDICATION.mjn3 for details.
  */
 
-/*  ATTENTION!   ATTENTION!   ATTENTION!   ATTENTION!   ATTENTION!
- *
- *  Besides uClibc, I'm using this code in my libc for elks, which is
- *  a 16-bit environment with a fairly limited compiler.  It would make
- *  things much easier for me if this file isn't modified unnecessarily.
- *  In particular, please put any new or replacement functions somewhere
- *  else, and modify the makefile to use your version instead.
- *  Thanks.  Manuel
- *
- *  ATTENTION!   ATTENTION!   ATTENTION!   ATTENTION!   ATTENTION! */
-
 /* June 15, 2002     Initial Notes:
  *
  * Note: It is assumed throught that time_t is either long or unsigned long.
@@ -97,8 +76,8 @@
  *   Bug reported by Arne Bernin <arne@alamut.de> in regards to freeswan.
  *
  * July 27, 2003  Adjust the struct tm extension field support.
- *   Change __tm_tzone back to a ptr and add the __tm_tzname[] buffer for
- *   __tm_tzone to point to.  This gets around complaints from g++.
+ *   Change __tm_zone back to a ptr and add the __tm_tzname[] buffer for
+ *   __tm_zone to point to.  This gets around complaints from g++.
  *  Who knows... it might even fix the PPC timezone init problem.
  *
  * July 29, 2003  Fix a bug in mktime behavior when tm_isdst was -1.
@@ -119,10 +98,38 @@
  *   unambiguous (not falling in the dst<->st transition region) both
  *   uClibc and glibc should produce the same result for mktime.
  *
+ * Oct 31, 2003 Kill the seperate __tm_zone and __tm_tzname[] and which
+ *   doesn't work if you want the memcpy the struct.  Sigh... I didn't
+ *   think about that.  So now, when the extensions are enabled, we
+ *   malloc space when necessary and keep the timezone names in a linked
+ *   list.
+ *
+ *   Fix a dst-related bug which resulted in use of uninitialized data.
+ *
+ * Nov 15, 2003 I forgot to update the thread locking in the last dst fix.
+ *
+ * Dec 14, 2003 Fix some dst issues in _time_mktime().
+ *   Normalize the tm_isdst value to -1, 0, or 1.
+ *   If no dst for this timezone, then reset tm_isdst to 0.
+ *
+ * May 7, 2004
+ *   Change clock() to allow wrapping.
+ *   Add timegm() function.
+ *   Make lookup_tzname() static (as it should have been).
+ *   Have strftime() get timezone information from the passed struct
+ *     for the %z and %Z conversions when using struct tm extensions.
+ *
+ * Jul 24, 2004
+ *   Fix 2 bugs in strftime related to glibc struct tm extensions.
+ *   1) Need to negate tm_gmtoff field value when used. (bug 336).
+ *   2) Deal with NULL ptr case for tm_zone field, which was causing
+ *      segfaults in both the NIST/PCTS tests and the Python 2.4.1
+ *      self-test suite.
+ *      NOTE: We set uninitialized timezone names to "???", and this
+ *            differs (intentionally) from glibc's behavior.
  */
 
 #define _GNU_SOURCE
-#define _STDIO_UTILITY
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
@@ -134,6 +141,9 @@
 #include <ctype.h>
 #include <langinfo.h>
 #include <locale.h>
+#include <bits/uClibc_uintmaxtostr.h>
+
+#include <bits/uClibc_mutex.h>
 
 #ifdef __UCLIBC_HAS_XLOCALE__
 #include <xlocale.h>
@@ -147,6 +157,22 @@
 #define TZNAME_MAX _POSIX_TZNAME_MAX
 #endif
 
+#if defined (L_tzset) || defined (L_localtime_r) || defined(L_strftime) || \
+	 defined(L__time_mktime) || defined(L__time_mktime_tzi) || \
+	 ((defined(L_strftime) || defined(L_strftime_l)) && \
+	  defined(__UCLIBC_HAS_XLOCALE__))
+
+void _time_tzset (int);
+
+#ifndef L__time_mktime
+
+ /* Jan 1, 2007 Z - tm = 0,0,0,1,0,107,1,0,0 */
+
+const static time_t new_rule_starts = 1167609600;
+
+#endif
+#endif
+
 /**********************************************************************/
 /* The era code is currently unfinished. */
 /*  #define ENABLE_ERA_CODE */
@@ -183,21 +209,7 @@ typedef struct {
 	char tzname[TZNAME_MAX+1];
 } rule_struct;
 
-#ifdef __UCLIBC_HAS_THREADS__
-
-#include <pthread.h>
-
-extern pthread_mutex_t _time_tzlock;
-
-#define TZLOCK		pthread_mutex_lock(&_time_tzlock)
-#define TZUNLOCK	pthread_mutex_unlock(&_time_tzlock)
-
-#else
-
-#define TZLOCK		((void) 0)
-#define TZUNLOCK	((void) 0)
-
-#endif
+__UCLIBC_MUTEX_EXTERN(_time_tzlock);
 
 extern rule_struct _time_tzinfo[2];
 
@@ -206,6 +218,13 @@ extern struct tm *_time_t2tm(const time_t *__restrict timer,
 
 extern time_t _time_mktime(struct tm *timeptr, int store_on_success);
 
+extern struct tm *__time_localtime_tzi(const time_t *__restrict timer,
+									   struct tm *__restrict result,
+									   rule_struct *tzi);
+
+extern time_t _time_mktime_tzi(struct tm *timeptr, int store_on_success,
+							   rule_struct *tzi);
+
 /**********************************************************************/
 #ifdef L_asctime
 
@@ -363,51 +382,63 @@ char *asctime_r(register const struct tm *__restrict ptm,
 
 #include <sys/times.h>
 
-/* Note: According to glibc...
- *    CAE XSH, Issue 4, Version 2: <time.h>
- *    The value of CLOCKS_PER_SEC is required to be 1 million on all
- *    XSI-conformant systems.
- */
-
 #ifndef __BCC__
 #if CLOCKS_PER_SEC != 1000000L
 #error unexpected value for CLOCKS_PER_SEC!
 #endif
 #endif
 
+#ifdef __UCLIBC_CLK_TCK_CONST
+# if __UCLIBC_CLK_TCK_CONST > CLOCKS_PER_SEC
+#  error __UCLIBC_CLK_TCK_CONST > CLOCKS_PER_SEC!
+# elif __UCLIBC_CLK_TCK_CONST < 1
+#  error __UCLIBC_CLK_TCK_CONST < 1!
+# endif
+#endif
+
+/* Note: SUSv3 notes
+ *
+ *   On XSI-conformant systems, CLOCKS_PER_SEC is defined to be one million.
+ *
+ *   The value returned by clock() may wrap around on some implementations.
+ *   For example, on a machine with 32-bit values for clock_t, it wraps
+ *   after 2147 seconds.
+ *
+ * This implies that we should bitwise and with LONG_MAX.
+ */
+
 clock_t clock(void)
 {
 	struct tms xtms;
 	unsigned long t;
 
 	times(&xtms);
+
 	t = ((unsigned long) xtms.tms_utime) + xtms.tms_stime;
 
 #ifndef __UCLIBC_CLK_TCK_CONST
-#error __UCLIBC_CLK_TCK_CONST not defined!
-#endif
 
-#undef CLK_TCK
-#define CLK_TCK __UCLIBC_CLK_TCK_CONST
+# error __UCLIBC_CLK_TCK_CONST not defined!
 
-#if CLK_TCK > CLOCKS_PER_SEC
-#error __UCLIBC_CLK_TCK_CONST > CLOCKS_PER_SEC!
-#elif CLK_TCK < 1
-#error __UCLIBC_CLK_TCK_CONST < 1!
-#endif
+#elif ((CLOCKS_PER_SEC % __UCLIBC_CLK_TCK_CONST) == 0)
+
+	/* CLOCKS_PER_SEC == k * __UCLIBC_CLK_TCK_CONST for some integer k >= 1. */
+	return ((t * (CLOCKS_PER_SEC/__UCLIBC_CLK_TCK_CONST)) & LONG_MAX);
 
-#if (CLK_TCK == CLOCKS_PER_SEC)
-	return (t <= LONG_MAX) ? t : -1;
-#elif (CLOCKS_PER_SEC % CLK_TCK) == 0
-	return (t <= (LONG_MAX / (CLOCKS_PER_SEC/CLK_TCK)))
-		? t * (CLOCKS_PER_SEC/CLK_TCK)
-		: -1;
 #else
-	return (t <= ((LONG_MAX / CLOCKS_PER_SEC) * CLK_TCK
-				  + ((LONG_MAX % CLOCKS_PER_SEC) * CLK_TCK) / CLOCKS_PER_SEC))
-		? (((t / CLK_TCK) * CLOCKS_PER_SEC)
-		   + (((t % CLK_TCK) * CLOCKS_PER_SEC) / CLK_TCK))
-		: -1;
+
+	/* Unlike the previous case, the scaling factor is not an integer.
+	 * So when tms_utime, tms_stime, or their sum wraps, some of the
+	 * "visible" bits in the return value are affected.  Nothing we
+	 * can really do about this though other than handle tms_utime and
+	 * tms_stime seperately and then sum.  But since that doesn't really
+	 * buy us much, we don't bother. */
+
+	return ((((t / __UCLIBC_CLK_TCK_CONST) * CLOCKS_PER_SEC)
+			 + ((((t % __UCLIBC_CLK_TCK_CONST) * CLOCKS_PER_SEC)
+				 / __UCLIBC_CLK_TCK_CONST))
+			 ) & LONG_MAX);
+
 #endif
 }
 
@@ -512,6 +543,65 @@ struct tm *localtime(const time_t *timer)
 /**********************************************************************/
 #ifdef L_localtime_r
 
+struct tm *localtime_r(register const time_t *__restrict timer,
+					   register struct tm *__restrict result)
+{
+	__UCLIBC_MUTEX_LOCK(_time_tzlock);
+
+	_time_tzset(*timer < new_rule_starts);
+
+	__time_localtime_tzi(timer, result, _time_tzinfo);
+
+	__UCLIBC_MUTEX_UNLOCK(_time_tzlock);
+
+	return result;
+}
+
+#endif
+/**********************************************************************/
+#ifdef L__time_localtime_tzi
+
+#ifdef __UCLIBC_HAS_TM_EXTENSIONS__
+
+struct ll_tzname_item;
+
+typedef struct ll_tzname_item {
+	struct ll_tzname_item *next;
+	char tzname[TZNAME_MAX+1];
+} ll_tzname_item_t;
+
+static ll_tzname_item_t ll_tzname[] = {
+	{ ll_tzname + 1, "UTC" },	/* Always 1st. */
+	{ NULL, "???" }		  /* Always 2nd. (invalid or out-of-memory) */
+};
+
+static const char *lookup_tzname(const char *key)
+{
+	ll_tzname_item_t *p;
+
+	for (p=ll_tzname ; p ; p=p->next) {
+		if (!strcmp(p->tzname, key)) {
+			return p->tzname;
+		}
+	}
+
+	/* Hmm... a new name. */
+	if (strnlen(key, TZNAME_MAX+1) < TZNAME_MAX+1) { /* Verify legal length */
+		if ((p = malloc(sizeof(ll_tzname_item_t))) != NULL) {
+			/* Insert as 3rd item in the list. */
+			p->next = ll_tzname[1].next;
+			ll_tzname[1].next = p;
+			strcpy(p->tzname, key);
+			return p->tzname;
+		}
+	}
+
+	/* Either invalid or couldn't alloc. */
+	return ll_tzname[1].tzname;
+}
+
+#endif /* __UCLIBC_HAS_TM_EXTENSIONS__ */
+
 static const unsigned char day_cor[] = { /* non-leap */
 	31, 31, 34, 34, 35, 35, 36, 36, 36, 37, 37, 38, 38
 /* 	 0,  0,  3,  3,  4,  4,  5,  5,  5,  6,  6,  7,  7 */
@@ -520,9 +610,9 @@ static const unsigned char day_cor[] = { /* non-leap */
 
 /* Note: timezone locking is done by localtime_r. */
 
-static int tm_isdst(register const struct tm *__restrict ptm)
+static int tm_isdst(register const struct tm *__restrict ptm,
+					register rule_struct *r)
 {
-	register rule_struct *r = _time_tzinfo;
 	long sec;
 	int i, isdst, isleap, day, day0, monlen, mday;
 	int oday;					/* Note: oday can be uninitialized. */
@@ -593,21 +683,18 @@ static int tm_isdst(register const struct tm *__restrict ptm)
 	return (isdst & 1);
 }
 
-struct tm *localtime_r(register const time_t *__restrict timer,
-					   register struct tm *__restrict result)
+struct tm *__time_localtime_tzi(register const time_t *__restrict timer,
+								register struct tm *__restrict result,
+								rule_struct *tzi)
 {
 	time_t x[1];
 	long offset;
 	int days, dst;
 
-	TZLOCK;
-
-	tzset();
-
 	dst = 0;
 	do {
 		days = -7;
-		offset = 604800L - _time_tzinfo[dst].gmt_offset;
+		offset = 604800L - tzi[dst].gmt_offset;
 		if (*timer > (LONG_MAX - 604800L)) {
 			days = -days;
 			offset = -offset;
@@ -617,13 +704,11 @@ struct tm *localtime_r(register const time_t *__restrict timer,
 		_time_t2tm(x, days, result);
 		result->tm_isdst = dst;
 #ifdef __UCLIBC_HAS_TM_EXTENSIONS__
-		result->tm_gmtoff = - _time_tzinfo[dst].gmt_offset;
-		result->tm_zone = result->__tm_tzname;
-		strcpy(result->__tm_tzname, _time_tzinfo[dst].tzname);
+		result->tm_gmtoff = - tzi[dst].gmt_offset;
+		result->tm_zone = lookup_tzname(tzi[dst].tzname);
 #endif /* __UCLIBC_HAS_TM_EXTENSIONS__ */
-	} while ((++dst < 2) && (result->tm_isdst = tm_isdst(result)) != 0);
-
-	TZUNLOCK;
+	} while ((++dst < 2)
+			 && ((result->tm_isdst = tm_isdst(result, tzi)) != 0));
 
 	return result;
 }
@@ -641,6 +726,20 @@ time_t mktime(struct tm *timeptr)
 	return  _time_mktime(timeptr, 1);
 }
 
+#endif
+/**********************************************************************/
+#ifdef L_timegm
+/* Like `mktime' but timeptr represents Universal Time, not local time. */
+
+time_t timegm(struct tm *timeptr)
+{
+	rule_struct gmt_tzinfo[2];
+
+	memset(gmt_tzinfo, 0, sizeof(gmt_tzinfo));
+	strcpy(gmt_tzinfo[0].tzname, "GMT"); /* Match glibc behavior here. */
+
+	return  _time_mktime_tzi(timeptr, 1, gmt_tzinfo);
+}
 
 #endif
 /**********************************************************************/
@@ -860,7 +959,9 @@ size_t __XL(strftime)(char *__restrict s, size_t maxsize,
 	long tzo;
 	register const char *p;
 	register const char *o;
+#ifndef __UCLIBC_HAS_TM_EXTENSIONS__
 	const rule_struct *rsp;
+#endif
 	const char *stack[MAX_PUSH];
 	size_t count;
 	size_t o_count;
@@ -871,7 +972,8 @@ size_t __XL(strftime)(char *__restrict s, size_t maxsize,
 	unsigned char mod;
 	unsigned char code;
 
-	tzset();					/* We'll, let's get this out of the way. */
+	/* We'll, let's get this out of the way. */
+	_time_tzset(_time_mktime((struct tm *) timeptr, 0) < new_rule_starts);
 
 	lvl = 0;
 	p = format;
@@ -940,7 +1042,7 @@ size_t __XL(strftime)(char *__restrict s, size_t maxsize,
 			goto LOOP;
 		}
 
-		o = spec + 26;		/* set to "????" */
+		o = ((const char *) spec) + 26;	/* set to "????" */
 		if ((code & MASK_SPEC) == CALC_SPEC) {
 
 			if (*p == 's') {
@@ -974,15 +1076,40 @@ size_t __XL(strftime)(char *__restrict s, size_t maxsize,
 					goto OUTPUT;
 				}
 
-				TZLOCK;
+#ifdef __UCLIBC_HAS_TM_EXTENSIONS__
+
+#define RSP_TZNAME		timeptr->tm_zone
+#define RSP_GMT_OFFSET	(-timeptr->tm_gmtoff)
+
+#else
+
+#define RSP_TZNAME		rsp->tzname
+#define RSP_GMT_OFFSET	rsp->gmt_offset
+
+				__UCLIBC_MUTEX_LOCK(_time_tzlock);
 
 				rsp = _time_tzinfo;
 				if (timeptr->tm_isdst > 0) {
 					++rsp;
 				}
+#endif
 
 				if (*p == 'Z') {
-					o = rsp->tzname;
+					o = RSP_TZNAME;
+#ifdef __UCLIBC_HAS_TM_EXTENSIONS__
+					/* Sigh... blasted glibc extensions.  Of course we can't
+					 * count on the pointer being valid.  Best we can do is
+					 * handle NULL, which looks to be all that glibc does.
+					 * At least that catches the memset() with 0 case.
+					 * NOTE: We handle this case differently than glibc!
+					 * It uses system timezone name (based on tm_isdst) in this
+					 * case... although it always seems to use the embedded
+					 * tm_gmtoff value.  What we'll do instead is treat the
+					 * timezone name as unknown/invalid and return "???". */
+					if (!o) {
+							o = "???";
+					}
+#endif
 					assert(o != NULL);
 #if 0
 					if (!o) {	/* PARANOIA */
@@ -990,15 +1117,17 @@ size_t __XL(strftime)(char *__restrict s, size_t maxsize,
 					}
 #endif
 					o_count = SIZE_MAX;
-					TZUNLOCK;
+/* 					RSP_TZUNLOCK; */
+#ifdef __UCLIBC_HAS_TM_EXTENSIONS__
 					goto OUTPUT;
+#endif
 				} else {		/* z */
 					*s = '+';
-					if ((tzo = -rsp->gmt_offset) < 0) {
+					if ((tzo = -RSP_GMT_OFFSET) < 0) {
 						tzo = -tzo;
 						*s = '-';
 					}
-					TZUNLOCK;
+/* 					RSP_TZUNLOCK; */
 					++s;
 					--count;
 
@@ -1007,6 +1136,13 @@ size_t __XL(strftime)(char *__restrict s, size_t maxsize,
 			
 					i = 16 + 6;	/* 0-fill, width = 4 */
 				}
+#ifdef __UCLIBC_HAS_TM_EXTENSIONS__
+#else
+				__UCLIBC_MUTEX_UNLOCK(_time_tzlock);
+				if (*p == 'Z') {
+					goto OUTPUT;
+				}
+#endif
 			} else {
 				/* TODO: don't need year for U, W */
 				for (i=0 ; i < 3 ; i++) {
@@ -1304,7 +1440,9 @@ char *__XL(strptime)(const char *__restrict buf, const char *__restrict format,
 
 			i = 0;
 			do {				/* Store the values into tm. */
-				((int *) tm)[i] = fields[i];
+				if (fields[i] != INT_MIN) {
+					((int *) tm)[i] = fields[i];
+				}
 			} while (++i < 8);
 
 			return (char *) buf; /* Success. */
@@ -1523,7 +1661,9 @@ static const char vals[] = {
 	6,  0,  0,					/* Note: overloaded for non-M non-J case... */
 	0, 1, 0,					/* J */
 	',', 'M',      '4', '.', '1', '.', '0',
-	',', 'M', '1', '0', '.', '5', '.', '0', 0
+	',', 'M', '1', '0', '.', '5', '.', '0', 0,
+	',', 'M',      '3', '.', '2', '.', '0',
+	',', 'M', '1', '1', '.', '1', '.', '0', 0
 };
 
 #define TZ    vals
@@ -1531,15 +1671,14 @@ static const char vals[] = {
 #define RANGE (vals + 7)
 #define RULE  (vals + 11 - 1)
 #define DEFAULT_RULES (vals + 22)
+#define DEFAULT_2007_RULES (vals + 38)
 
 /* Initialize to UTC. */
 int daylight = 0;
 long timezone = 0;
 char *tzname[2] = { (char *) UTC, (char *) (UTC-1) };
 
-#ifdef __UCLIBC_HAS_THREADS__
-pthread_mutex_t _time_tzlock = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
-#endif
+__UCLIBC_MUTEX_INIT(_time_tzlock, PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP);
 
 rule_struct _time_tzinfo[2];
 
@@ -1654,6 +1793,11 @@ static char *read_TZ_file(char *buf)
 #endif /* __UCLIBC_HAS_TZ_FILE__ */
 
 void tzset(void)
+{
+    _time_tzset((time(NULL)) < new_rule_starts);
+}
+
+void _time_tzset(int use_old_rules)
 {
 	register const char *e;
 	register char *s;
@@ -1669,7 +1813,7 @@ void tzset(void)
 	static char oldval[TZ_BUFLEN]; /* BSS-zero'd. */
 #endif /* __UCLIBC_HAS_TZ_CACHING__ */
 
-	TZLOCK;
+	__UCLIBC_MUTEX_LOCK(_time_tzlock);
 
 	e = getenv(TZ);				/* TZ env var always takes precedence. */
 
@@ -1698,13 +1842,8 @@ void tzset(void)
 #ifdef __UCLIBC_HAS_TZ_CACHING__
 		*oldval = 0;			/* Set oldval to an empty string. */
 #endif /* __UCLIBC_HAS_TZ_CACHING__ */
-		s = _time_tzinfo[0].tzname;
-		*s = 'U';
-		*++s = 'T';
-		*++s = 'C';
-		*++s =
-		*_time_tzinfo[1].tzname = 0;
-		_time_tzinfo[0].gmt_offset = 0;
+		memset(_time_tzinfo, 0, 2*sizeof(rule_struct));
+		strcpy(_time_tzinfo[0].tzname, UTC);
 		goto DONE;
 	}
 
@@ -1774,6 +1913,7 @@ void tzset(void)
 	new_rules[count].gmt_offset = off;
 
 	if (!count) {
+		new_rules[1].gmt_offset = off; /* Shouldn't be needed... */
 		if (*e) {
 			++count;
 			goto LOOP;
@@ -1781,7 +1921,15 @@ void tzset(void)
 	} else {					/* OK, we have dst, so get some rules. */
 		count = 0;
 		if (!*e) {				/* No rules so default to US rules. */
-			e = DEFAULT_RULES;
+		        e = use_old_rules ? DEFAULT_RULES : DEFAULT_2007_RULES;
+#ifdef DEBUG_TZSET			
+			if (e == DEFAULT_RULES)
+			    printf("tzset: Using old rules.\n");
+			else if (e == DEFAULT_2007_RULES)
+			    printf("tzset: Using new rules\n");
+			else
+			    printf("tzset: Using undefined rules\n");
+#endif /* DEBUG_TZSET */
 		}
 
 		do {
@@ -1839,8 +1987,10 @@ void tzset(void)
 	daylight = !!_time_tzinfo[1].tzname[0];
 	timezone = _time_tzinfo[0].gmt_offset;
 
+#if defined(__UCLIBC_HAS_TZ_FILE__) || defined(__UCLIBC_HAS_TZ_CACHING__)
  FAST_DONE:
-	TZUNLOCK;
+#endif
+	__UCLIBC_MUTEX_UNLOCK(_time_tzlock);
 }
 
 #endif
@@ -1889,6 +2039,10 @@ static const unsigned char days[] = {
 	    29,
 };
 
+#ifdef __UCLIBC_HAS_TM_EXTENSIONS__
+static const char utc_string[] = "UTC";
+#endif
+
 /* Notes:
  * If time_t is 32 bits, then no overflow is possible.
  * It time_t is > 32 bits, this needs to be adjusted to deal with overflow.
@@ -2018,14 +2172,7 @@ struct tm *_time_t2tm(const time_t *__restrict timer,
 	p[4] = 0;					/* result[8] .. tm_isdst */
 #ifdef __UCLIBC_HAS_TM_EXTENSIONS__
 	result->tm_gmtoff = 0;
-	result->tm_zone = result->__tm_tzname;
-	{
-		register char *s = result->__tm_tzname;
-		*s = 'U';
-		*++s = 'T';
-		*++s = 'C';
-		*++s = 0;
-	}
+	result->tm_zone = utc_string;
 #endif /* __UCLIBC_HAS_TM_EXTENSIONS__ */
 
 	return result;
@@ -2041,12 +2188,32 @@ struct tm __time_tm;	/* Global shared by gmtime() and localtime(). */
 /**********************************************************************/
 #ifdef L__time_mktime
 
+time_t _time_mktime(struct tm *timeptr, int store_on_success)
+{
+	time_t t;
+
+	__UCLIBC_MUTEX_LOCK(_time_tzlock);
+
+	tzset();
+
+	t = _time_mktime_tzi(timeptr, store_on_success, _time_tzinfo);
+
+	__UCLIBC_MUTEX_UNLOCK(_time_tzlock);
+
+	return t;
+}
+
+#endif
+/**********************************************************************/
+#ifdef L__time_mktime_tzi
+
 static const unsigned char vals[] = {
 	31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31, /* non-leap */
 	    29,
 };
 
-time_t _time_mktime(struct tm *timeptr, int store_on_success)
+time_t _time_mktime_tzi(struct tm *timeptr, int store_on_success,
+						rule_struct *tzi)
 {
 #ifdef __BCC__
 	long days, secs;
@@ -2060,12 +2227,16 @@ time_t _time_mktime(struct tm *timeptr, int store_on_success)
 	register const unsigned char *s;
 	int d, default_dst;
 
-	tzset();
-
 	memcpy(p, timeptr, sizeof(struct tm));
 
-	if ((default_dst = p[8]) < 0) {
-		default_dst = 1;		/* Assume advancing */
+	if (!tzi[1].tzname[0]) { /* No dst in this timezone, */
+		p[8] = 0;				/* so set tm_isdst to 0. */
+	}
+
+	default_dst = 0;
+	if (p[8]) {					/* Either dst or unknown? */
+		default_dst = 1;		/* Assume advancing (even if unknown). */
+		p[8] = ((p[8] > 0) ? 1 : -1); /* Normalize so abs() <= 1. */
 	}
 
 	d = 400;
@@ -2092,13 +2263,13 @@ time_t _time_mktime(struct tm *timeptr, int store_on_success)
 		--d;
 	}
 
-	TZLOCK;
+	_time_tzset (x.tm_year < 2007);	/* tm_year was expanded above */
 
 #ifdef __BCC__
 	d = p[5] - 1;
 	days = -719163L + ((long)d)*365 + ((d/4) - (d/100) + (d/400) + p[3] + p[7]);
 	secs = p[0] + 60*( p[1] + 60*((long)(p[2])) )
-		+ _time_tzinfo[default_dst].gmt_offset;
+		+ tzi[default_dst].gmt_offset;
  DST_CORRECT:
 	if (secs < 0) {
 		secs += 120009600L;
@@ -2113,7 +2284,7 @@ time_t _time_mktime(struct tm *timeptr, int store_on_success)
 	d = p[5] - 1;
 	d = -719163L + d*365 + (d/4) - (d/100) + (d/400);
 	secs = p[0]
-		+ _time_tzinfo[default_dst].gmt_offset
+		+ tzi[default_dst].gmt_offset
 		+ 60*( p[1]
 			   + 60*(p[2]
 					 + 24*(((146073L * ((long long)(p[6])) + d)
@@ -2131,7 +2302,7 @@ time_t _time_mktime(struct tm *timeptr, int store_on_success)
 	d = ((struct tm *)p)->tm_isdst;
 	t = secs;
 
-	localtime_r(&t, (struct tm *)p);
+	__time_localtime_tzi(&t, (struct tm *)p, tzi);
 
 	if (t == ((time_t)(-1))) {	/* Remember, time_t can be unsigned. */
 	    goto DONE;
@@ -2141,8 +2312,8 @@ time_t _time_mktime(struct tm *timeptr, int store_on_success)
 #ifdef __BCC__
 		secs -= (days * 86400L);
 #endif
-		secs += (_time_tzinfo[1-default_dst].gmt_offset
-				 - _time_tzinfo[default_dst].gmt_offset);
+		secs += (tzi[1-default_dst].gmt_offset
+				 - tzi[default_dst].gmt_offset);
 		goto DST_CORRECT;
 	}
 
@@ -2153,11 +2324,41 @@ time_t _time_mktime(struct tm *timeptr, int store_on_success)
 
 
  DONE:
-	TZUNLOCK;
-
 	return t;
 }
 
+#endif
+/**********************************************************************/
+#if defined(L_wcsftime) || defined(L_wcsftime_l)
+
+#if defined(__UCLIBC_HAS_XLOCALE__) && !defined(__UCLIBC_DO_XLOCALE)
+
+extern size_t __wcsftime_l (wchar_t *__restrict __s, size_t __maxsize,
+			  __const wchar_t *__restrict __format,
+			  __const struct tm *__restrict __timeptr,
+			  __locale_t __loc) __THROW;
+
+size_t wcsftime(wchar_t *__restrict s, size_t maxsize,
+				const wchar_t *__restrict format,
+				const struct tm *__restrict timeptr)
+{
+	return __wcsftime_l(s, maxsize, format, timeptr, __UCLIBC_CURLOCALE);
+}
+
+#else  /* defined(__UCLIBC_HAS_XLOCALE__) && !defined(__UCLIBC_DO_XLOCALE) */
+
+size_t __XL(wcsftime)(wchar_t *__restrict s, size_t maxsize,
+					  const wchar_t *__restrict format,
+					  const struct tm *__restrict timeptr   __LOCALE_PARAM )
+{
+#warning wcsftime always fails
+	return 0;					/* always fail */
+}
+
+__XL_ALIAS(wcsftime)
+
+#endif /* defined(__UCLIBC_HAS_XLOCALE__) && !defined(__UCLIBC_DO_XLOCALE) */
+
 #endif
 /**********************************************************************/
 #ifdef L_dysize
@@ -2170,7 +2371,3 @@ int dysize(int year)
 
 #endif
 /**********************************************************************/
-/* Like `mktime', but for TP represents Universal Time, not local time.  */
-/* time_t timegm(struct tm *tp) */
-
-
