@@ -1,18 +1,8 @@
-/*  Copyright (C) 2004     Manuel Novoa III
+/* Copyright (C) 2004       Manuel Novoa III    <mjn3@codepoet.org>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License as published by the Free Software Foundation; either
- *  version 2 of the License, or (at your option) any later version.
+ * GNU Library General Public License (LGPL) version 2 or later.
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public
- *  License along with this library; if not, write to the Free
- *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Dedicated to Toni.  See uClibc/DEDICATION.mjn3 for details.
  */
 
 /* Jan 1, 2004
@@ -24,6 +14,7 @@
  *   Fix failure exit code for failed execve().
  */
 
+#warning hmm... susv3 says "Pipe streams are byte-oriented."
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -31,6 +22,8 @@
 #include <unistd.h>
 #include <sys/wait.h>
 
+#include <bits/uClibc_mutex.h>
+
 /* uClinux-2.0 has vfork, but Linux 2.0 doesn't */
 #include <sys/syscall.h>
 #if ! defined __NR_vfork
@@ -39,24 +32,11 @@
 # define VFORK_UNLOCK	((void) 0)
 #endif
 
-#ifdef __UCLIBC_HAS_THREADS__
-#include <pthread.h>
-static pthread_mutex_t mylock = PTHREAD_MUTEX_INITIALIZER;
-# define LOCK			__pthread_mutex_lock(&mylock)
-# define UNLOCK			__pthread_mutex_unlock(&mylock);
-#else
-# define LOCK			((void) 0)
-# define UNLOCK			((void) 0)
-#endif      
+__UCLIBC_MUTEX_STATIC(mylock, PTHREAD_MUTEX_INITIALIZER);
 
 #ifndef VFORK_LOCK
-# define VFORK_LOCK		LOCK
-# define VFORK_UNLOCK	UNLOCK
-#endif
-
-/* Temporarily support old stdio code. */
-#ifndef __MASK_READING
-#define __filedes		filedes
+# define VFORK_LOCK		__UCLIBC_MUTEX_LOCK(mylock)
+# define VFORK_UNLOCK	__UCLIBC_MUTEX_UNLOCK(mylock)
 #endif
 
 struct popen_list_item {
@@ -133,10 +113,10 @@ FILE *popen(const char *command, const char *modes)
 	if (pid > 0) {				/* Parent of vfork... */
 		pi->pid = pid;
 		pi->f = fp;
-		LOCK;
+		__UCLIBC_MUTEX_LOCK(mylock);
 		pi->next = popen_list;
 		popen_list = pi;
-		UNLOCK;
+		__UCLIBC_MUTEX_UNLOCK(mylock);
 		
 		return fp;
 	}
@@ -151,6 +131,8 @@ FILE *popen(const char *command, const char *modes)
 	return NULL;
 }
 
+#warning is pclose correct wrt the new mutex semantics?
+
 int pclose(FILE *stream)
 {
 	struct popen_list_item *p;
@@ -159,7 +141,7 @@ int pclose(FILE *stream)
 
 	/* First, find the list entry corresponding to stream and remove it
 	 * from the list.  Set p to the list item (NULL if not found). */
-	LOCK;
+	__UCLIBC_MUTEX_LOCK(mylock);
 	if ((p = popen_list) != NULL) {
 		if (p->f == stream) {
 			popen_list = p->next;
@@ -178,7 +160,7 @@ int pclose(FILE *stream)
 			} while (1);
 		}
 	}
-	UNLOCK;
+	__UCLIBC_MUTEX_UNLOCK(mylock);
 
 	if (p) {
 		pid = p->pid;			/* Save the pid we need */
